@@ -46,6 +46,8 @@ function configure_fwaas_v2() {
     neutron_fwaas_configure_driver fwaas_v2
     iniset_multiline $Q_L3_CONF_FILE fwaas agent_version v2
     iniset_multiline $Q_L3_CONF_FILE fwaas driver $FWAAS_DRIVER_V2
+    iniset $NEUTRON_CORE_PLUGIN_CONF fwaas firewall_l2_driver $FW_L2_DRIVER
+    iniset $NEUTRON_CORE_PLUGIN_CONF agent extensions fwaas_v2
 }
 
 function neutron_fwaas_generate_config_files {
