@@ -15,17 +15,17 @@
 
 import operator
 
-from neutron_lib.plugins import directory
-
-from neutron.callbacks import events
-from neutron.callbacks import registry
-from neutron.callbacks import resources
-from neutron.db import agents_db
 from neutron.db import common_db_mixin as base_db
-from neutron.db import l3_agentschedulers_db as l3_sch_db
-from neutron.extensions import l3
+from neutron.db.models import agent as agent_model
+from neutron.db.models import l3agent as l3agent_model
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as nl_constants
 from neutron_lib.db import model_base
+from neutron_lib.exceptions import firewall_v1 as f_exc
+from neutron_lib.exceptions import l3
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import uuidutils
@@ -111,19 +111,19 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         try:
             return self._get_by_id(context, Firewall, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallNotFound(firewall_id=id)
+            raise f_exc.FirewallNotFound(firewall_id=id)
 
     def _get_firewall_policy(self, context, id):
         try:
             return self._get_by_id(context, FirewallPolicy, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallPolicyNotFound(firewall_policy_id=id)
+            raise f_exc.FirewallPolicyNotFound(firewall_policy_id=id)
 
     def _get_firewall_rule(self, context, id):
         try:
             return self._get_by_id(context, FirewallRule, id)
         except exc.NoResultFound:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=id)
+            raise f_exc.FirewallRuleNotFound(firewall_rule_id=id)
 
     def _make_firewall_dict(self, fw, fields=None):
         res = {'id': fw['id'],
@@ -198,9 +198,9 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
     def _check_firewall_rule_conflict(self, fwr_db, fwp_db):
         if not fwr_db['shared']:
             if fwr_db['tenant_id'] != fwp_db['tenant_id']:
-                raise fw_ext.FirewallRuleConflict(
+                raise f_exc.FirewallRuleConflict(
                     firewall_rule_id=fwr_db['id'],
-                    tenant_id=fwr_db['tenant_id'])
+                    project_id=fwr_db['tenant_id'])
 
     def _set_rules_for_policy(self, context, firewall_policy_db, fwp):
         rule_id_list = fwp['firewall_rules']
@@ -220,20 +220,20 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                     # If we find an invalid rule in the list we
                     # do not perform the update since this breaks
                     # the integrity of this list.
-                    raise fw_ext.FirewallRuleNotFound(
+                    raise f_exc.FirewallRuleNotFound(
                         firewall_rule_id=fwrule_id)
                 elif rules_dict[fwrule_id]['firewall_policy_id']:
                     if (rules_dict[fwrule_id]['firewall_policy_id'] !=
                             fwp_db['id']):
-                        raise fw_ext.FirewallRuleInUse(
+                        raise f_exc.FirewallRuleInUse(
                             firewall_rule_id=fwrule_id)
                 if 'shared' in fwp:
                     if fwp['shared'] and not rules_dict[fwrule_id]['shared']:
-                        raise fw_ext.FirewallRuleSharingConflict(
+                        raise f_exc.FirewallRuleSharingConflict(
                             firewall_rule_id=fwrule_id,
                             firewall_policy_id=fwp_db['id'])
                 elif fwp_db['shared'] and not rules_dict[fwrule_id]['shared']:
-                    raise fw_ext.FirewallRuleSharingConflict(
+                    raise f_exc.FirewallRuleSharingConflict(
                         firewall_rule_id=fwrule_id,
                         firewall_policy_id=fwp_db['id'])
             for fwr_db in rules_in_db:
@@ -253,7 +253,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             rules_in_db = fwp_db['firewall_rules']
             for fwr_db in rules_in_db:
                 if not fwr_db['shared']:
-                    raise fw_ext.FirewallPolicySharingConflict(
+                    raise f_exc.FirewallPolicySharingConflict(
                         firewall_rule_id=fwr_db['id'],
                         firewall_policy_id=fwp_db['id'])
 
@@ -296,7 +296,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         fwp_id = fw['firewall_policy_id']
         fwp = self._get_firewall_policy(context, fwp_id)
         if fw_tenant_id != fwp['tenant_id'] and not fwp['shared']:
-            raise fw_ext.FirewallPolicyConflict(firewall_policy_id=fwp_id)
+            raise f_exc.FirewallPolicyConflict(firewall_policy_id=fwp_id)
 
     def _validate_fwr_src_dst_ip_version(self, fwr):
         src_version = dst_version = None
@@ -308,12 +308,12 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         rule_ip_version = fwr.get('ip_version', None)
         if ((src_version and src_version != rule_ip_version) or
                 (dst_version and dst_version != rule_ip_version)):
-            raise fw_ext.FirewallIpAddressConflict()
+            raise f_exc.FirewallIpAddressConflict()
 
     def _validate_fwr_port_range(self, min_port, max_port):
         if int(min_port) > int(max_port):
             port_range = '%s:%s' % (min_port, max_port)
-            raise fw_ext.FirewallRuleInvalidPortValue(port=port_range)
+            raise f_exc.FirewallRuleInvalidPortValue(port=port_range)
 
     def _validate_fwr_protocol_parameters(self, fwr):
         protocol = fwr.get('protocol', None)
@@ -321,7 +321,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                             nl_constants.PROTO_NAME_UDP):
             if (fwr.get('source_port', None) or
                     fwr.get('destination_port', None)):
-                raise fw_ext.FirewallRuleInvalidICMPParameter(
+                raise f_exc.FirewallRuleInvalidICMPParameter(
                     param="Source, destination port")
 
     def create_firewall(self, context, firewall, status=None):
@@ -355,7 +355,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             self._validate_fw_parameters(context, fw, fw_db['tenant_id'])
             count = context.session.query(Firewall).filter_by(id=id).update(fw)
             if not count:
-                raise fw_ext.FirewallNotFound(firewall_id=id)
+                raise f_exc.FirewallNotFound(firewall_id=id)
         return self.get_firewall(context, id)
 
     def update_firewall_status(self, context, id, status, not_in=None):
@@ -379,7 +379,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             # firewall is active
             count = context.session.query(Firewall).filter_by(id=id).delete()
             if not count:
-                raise fw_ext.FirewallNotFound(firewall_id=id)
+                raise f_exc.FirewallNotFound(firewall_id=id)
 
     def get_firewall(self, context, id, fields=None):
         LOG.debug("get_firewall() called")
@@ -420,7 +420,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             if not fwp.get('shared', True) and fwp_db.firewalls:
                 for fw in fwp_db['firewalls']:
                     if fwp_db['tenant_id'] != fw['tenant_id']:
-                        raise fw_ext.FirewallPolicyInUse(
+                        raise f_exc.FirewallPolicyInUse(
                             firewall_policy_id=id)
             # check any existing rules are not shared
             if 'shared' in fwp and 'firewall_rules' not in fwp:
@@ -441,7 +441,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             # being used
             qry = context.session.query(Firewall)
             if qry.filter_by(firewall_policy_id=id).first():
-                raise fw_ext.FirewallPolicyInUse(firewall_policy_id=id)
+                raise f_exc.FirewallPolicyInUse(firewall_policy_id=id)
             else:
                 context.session.delete(fwp)
 
@@ -468,7 +468,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         self._validate_fwr_src_dst_ip_version(fwr)
         if not fwr['protocol'] and (fwr['source_port'] or
            fwr['destination_port']):
-            raise fw_ext.FirewallRuleWithPortWithoutProtocolInvalid()
+            raise f_exc.FirewallRuleWithPortWithoutProtocolInvalid()
         src_port_min, src_port_max = self._get_min_max_ports_from_range(
             fwr['source_port'])
         dst_port_min, dst_port_max = self._get_min_max_ports_from_range(
@@ -504,7 +504,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                                                fwr_db.firewall_policy_id)
             if 'shared' in fwr and not fwr['shared']:
                 if fwr_db['tenant_id'] != fwp_db['tenant_id']:
-                    raise fw_ext.FirewallRuleInUse(firewall_rule_id=id)
+                    raise f_exc.FirewallRuleInUse(firewall_rule_id=id)
         if 'source_port' in fwr:
             src_port_min, src_port_max = self._get_min_max_ports_from_range(
                 fwr['source_port'])
@@ -525,7 +525,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                 dport = fwr.get('destination_port_range_min',
                                 fwr_db['destination_port_range_min'])
                 if sport or dport:
-                    raise fw_ext.FirewallRuleWithPortWithoutProtocolInvalid()
+                    raise f_exc.FirewallRuleWithPortWithoutProtocolInvalid()
             fwr_db.update(fwr)
             if fwr_db.firewall_policy_id:
                 fwp_db.audited = False
@@ -536,7 +536,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         with context.session.begin(subtransactions=True):
             fwr = self._get_firewall_rule(context, id)
             if fwr.firewall_policy_id:
-                raise fw_ext.FirewallRuleInUse(firewall_rule_id=id)
+                raise f_exc.FirewallRuleInUse(firewall_rule_id=id)
             context.session.delete(fwr)
 
     def get_firewall_rule(self, context, id, fields=None):
@@ -557,7 +557,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
 
     def _validate_insert_remove_rule_request(self, id, rule_info):
         if not rule_info or 'firewall_rule_id' not in rule_info:
-            raise fw_ext.FirewallRuleInfoMissing()
+            raise f_exc.FirewallRuleInfoMissing()
 
     def insert_rule(self, context, id, rule_info):
         LOG.debug("insert_rule() called")
@@ -566,7 +566,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         insert_before = True
         ref_firewall_rule_id = None
         if not firewall_rule_id:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=None)
+            raise f_exc.FirewallRuleNotFound(firewall_rule_id=None)
         if 'insert_before' in rule_info:
             ref_firewall_rule_id = rule_info['insert_before']
         if not ref_firewall_rule_id and 'insert_after' in rule_info:
@@ -577,7 +577,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
             fwr_db = self._get_firewall_rule(context, firewall_rule_id)
             fwp_db = self._get_firewall_policy(context, id)
             if fwr_db.firewall_policy_id:
-                raise fw_ext.FirewallRuleInUse(firewall_rule_id=fwr_db['id'])
+                raise f_exc.FirewallRuleInUse(firewall_rule_id=fwr_db['id'])
             self._check_firewall_rule_conflict(fwr_db, fwp_db)
             if ref_firewall_rule_id:
                 # If reference_firewall_rule_id is set, the new rule
@@ -588,7 +588,7 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
                 ref_fwr_db = self._get_firewall_rule(
                     context, ref_firewall_rule_id)
                 if ref_fwr_db.firewall_policy_id != id:
-                    raise fw_ext.FirewallRuleNotAssociatedWithPolicy(
+                    raise f_exc.FirewallRuleNotAssociatedWithPolicy(
                         firewall_rule_id=ref_fwr_db['id'],
                         firewall_policy_id=id)
                 if insert_before:
@@ -610,11 +610,11 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
         self._validate_insert_remove_rule_request(id, rule_info)
         firewall_rule_id = rule_info['firewall_rule_id']
         if not firewall_rule_id:
-            raise fw_ext.FirewallRuleNotFound(firewall_rule_id=None)
+            raise f_exc.FirewallRuleNotFound(firewall_rule_id=None)
         with context.session.begin(subtransactions=True):
             fwr_db = self._get_firewall_rule(context, firewall_rule_id)
             if fwr_db.firewall_policy_id != id:
-                raise fw_ext.FirewallRuleNotAssociatedWithPolicy(
+                raise f_exc.FirewallRuleNotAssociatedWithPolicy(
                     firewall_rule_id=fwr_db['id'],
                     firewall_policy_id=id)
             return self._process_rule_for_policy(context, id, fwr_db, None)
@@ -622,11 +622,11 @@ class Firewall_db_mixin(fw_ext.FirewallPluginBase, base_db.CommonDbMixin):
     def get_firewall_tenant_ids_on_host(self, context, host):
         query = context.session.query(Firewall.tenant_id)
         query = query.join(fw_r_ins_db.FirewallRouterAssociation)
-        query = query.join(l3_sch_db.RouterL3AgentBinding,
-                           l3_sch_db.RouterL3AgentBinding.router_id ==
+        query = query.join(l3agent_model.RouterL3AgentBinding,
+                           l3agent_model.RouterL3AgentBinding.router_id ==
                            fw_r_ins_db.FirewallRouterAssociation.router_id)
-        query = query.join(agents_db.Agent)
-        query = query.filter(agents_db.Agent.host == host)
+        query = query.join(agent_model.Agent)
+        query = query.filter(agent_model.Agent.host == host)
         query = query.distinct()
         return [item[0] for item in query]
 
