@@ -23,7 +23,6 @@ from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 
-from neutron_fwaas._i18n import _, _LE
 from neutron_fwaas.common import fwaas_constants
 from neutron_fwaas.common import resources as f_resources
 from neutron_fwaas.services.firewall.agents import firewall_agent_api as api
@@ -110,8 +109,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             fwaas_plugin_configured = (fwaas_constants.FIREWALL
                                        in self.neutron_service_plugins)
             if fwaas_plugin_configured and not self.fwaas_enabled:
-                msg = _("FWaaS plugin is configured in the server side, but "
-                        "FWaaS is disabled in L3-agent.")
+                msg = ("FWaaS plugin is configured in the server side, but "
+                       "FWaaS is disabled in L3-agent.")
                 LOG.error(msg)
                 raise SystemExit(1)
             self.fwaas_enabled = self.fwaas_enabled and fwaas_plugin_configured
@@ -128,8 +127,7 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
 
     @property
     def _local_namespaces(self):
-        root_ip = ip_lib.IPWrapper()
-        local_ns_list = root_ip.get_namespaces()
+        local_ns_list = ip_lib.list_network_namespaces()
         return local_ns_list
 
     def _has_port_insertion_fields(self, firewall_group):
@@ -207,8 +205,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 self.fwplugin_rpc.firewall_group_deleted(
                     ctx, firewall_group['id'])
             except fw_ext.FirewallInternalDriverError:
-                msg = _LE("FWaaS driver error on %(status)s "
-                          "for firewall group: %(fwg_id)s")
+                msg = ("FWaaS driver error on %(status)s "
+                       "for firewall group: %(fwg_id)s")
                 LOG.exception(msg, {'status': firewall_group['status'],
                                     'fwg_id': firewall_group['id']})
                 self.fwplugin_rpc.set_firewall_group_status(
@@ -227,9 +225,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 self.fwaas_driver.update_firewall_group(
                     self.conf.agent_mode, port_list, firewall_group)
             except fw_ext.FirewallInternalDriverError:
-                msg = _LE("FWaaS driver error on %(status)s for firewall "
-                          "group: "
-                      "%(fwg_id)s")
+                msg = ("FWaaS driver error on %(status)s for firewall "
+                       "group: %(fwg_id)s")
                 LOG.exception(msg, {'status': firewall_group['status'],
                                     'fwg_id': firewall_group['id']})
                 status = nl_constants.ERROR
@@ -293,8 +290,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         try:
             self._process_router_update(new_router)
         except Exception:
-            LOG.exception(_LE("FWaaS router add RPC info call failed for %s"),
-                    new_router['id'])
+            LOG.exception("FWaaS router add RPC info call failed for %s",
+                          new_router['id'])
             self.services_sync_needed = True
 
     def update_router(self, context, updated_router):
@@ -309,8 +306,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
         except Exception:
             #TODO(njohnston): This repr should be replaced.
             LOG.exception(
-                    _LE("FWaaS router update RPC info call failed for %s"),
-                    repr(updated_router))
+                "FWaaS router update RPC info call failed for %s",
+                repr(updated_router))
             self.services_sync_needed = True
 
     def delete_router(self, context, new_router):
@@ -349,7 +346,7 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                                                    self.host)
             self.services_sync_needed = False
         except Exception:
-            LOG.exception(_LE("Failed FWaaS process services sync."))
+            LOG.exception("Failed FWaaS process services sync.")
             self.services_sync_needed = True
 
     @log_helpers.log_method_call
@@ -380,8 +377,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                                                     ports_for_fwg,
                                                     firewall_group)
         except fw_ext.FirewallInternalDriverError:
-            msg = _LE("FWaaS driver error in create_firewall_group "
-                      "for firewall group: %(fwg_id)s")
+            msg = ("FWaaS driver error in create_firewall_group "
+                   "for firewall group: %(fwg_id)s")
             LOG.exception(msg, {'fwg_id': firewall_group['id']})
             status = nl_constants.ERROR
 
@@ -390,8 +387,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             self.fwplugin_rpc.set_firewall_group_status(context,
                     firewall_group['id'], status)
         except Exception:
-            msg = _LE("FWaaS RPC failure in create_firewall_group "
-                      "for firewall group: %(fwg_id)s")
+            msg = ("FWaaS RPC failure in create_firewall_group "
+                   "for firewall group: %(fwg_id)s")
             LOG.exception(msg, {'fwg_id': firewall_group['id']})
             self.services_sync_needed = True
 
@@ -405,12 +402,21 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
 
         # Get the list of in-namespace ports from which to delete the firewall
         # group.
-        ports_for_fwg = self._get_firewall_group_ports(context, firewall_group,
-                to_delete=True, require_new_plugin=True)
+        del_fwg_ports = self._get_firewall_group_ports(
+            context, firewall_group, to_delete=True, require_new_plugin=True)
+        add_fwg_ports = self._get_firewall_group_ports(context, firewall_group)
+
+        port_ids = (firewall_group.get('del-port-ids') +
+                    firewall_group.get('add-port-ids'))
+
+        if port_ids and not (del_fwg_ports or add_fwg_ports):
+            LOG.debug("All ports are not router port."
+                      "No need to update firewall driver.")
+            return
 
         # Remove firewall group from ports if requested.
-        if ports_for_fwg:
-            fw_ports = [p for ri_ports in ports_for_fwg for p in ri_ports[1]]
+        if del_fwg_ports:
+            fw_ports = [p for ri_port in del_fwg_ports for p in ri_port[1]]
             LOG.debug("Update (delete) firewall group %(fwg_id)s on ports: "
                       "%(ports)s",
                       {'fwg_id': firewall_group['id'],
@@ -429,22 +435,20 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             # Call the driver.
             try:
                 self.fwaas_driver.delete_firewall_group(self.conf.agent_mode,
-                                                        ports_for_fwg,
+                                                        del_fwg_ports,
                                                         firewall_group)
             except fw_ext.FirewallInternalDriverError:
-                msg = _LE("FWaaS driver error in update_firewall_group "
-                          "(add) for firewall group: %s")
+                msg = ("FWaaS driver error in update_firewall_group "
+                       "(add) for firewall group: %s")
                 LOG.exception(msg, firewall_group['id'])
                 status = nl_constants.ERROR
 
         # Handle the add router and/or rule, policy, firewall group attribute
         # updates.
         if status not in (nl_constants.ERROR, nl_constants.INACTIVE):
-            ports_for_fwg = self._get_firewall_group_ports(context,
-                    firewall_group)
-            if ports_for_fwg:
-                fw_ports = [p for ri_ports in ports_for_fwg
-                            for p in ri_ports[1]]
+            if add_fwg_ports:
+                fw_ports = [p for ri_port in add_fwg_ports
+                            for p in ri_port[1]]
                 LOG.debug("Update (create) firewall group %(fwg_id)s on "
                           "ports: %(ports)s",
                           {'fwg_id': firewall_group['id'],
@@ -460,11 +464,11 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 # Call the driver.
                 try:
                     self.fwaas_driver.update_firewall_group(
-                            self.conf.agent_mode, ports_for_fwg,
+                            self.conf.agent_mode, add_fwg_ports,
                             firewall_group)
                 except fw_ext.FirewallInternalDriverError:
-                    msg = _LE("FWaaS driver error in update_firewall_group "
-                              "for firewall group: %s")
+                    msg = ("FWaaS driver error in update_firewall_group "
+                           "for firewall group: %s")
                     LOG.exception(msg, firewall_group['id'])
                     status = nl_constants.ERROR
             else:
@@ -475,9 +479,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
             self.fwplugin_rpc.set_firewall_group_status(context,
                     firewall_group['id'], status)
         except Exception:
-            LOG.exception(_LE("FWaaS RPC failure in update_firewall_group "
-                              "for firewall group: %(fwg_id)s"),
-                    {'fwg_id': firewall_group['id']})
+            LOG.exception("FWaaS RPC failure in update_firewall_group "
+                          "for firewall group: %s", firewall_group['id'])
             self.services_sync_needed = True
 
     @log_helpers.log_method_call
@@ -508,9 +511,8 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                                                     firewall_group)
         # Call the driver.
         except fw_ext.FirewallInternalDriverError:
-            LOG.exception(_LE("FWaaS driver error in delete_firewall_group "
-                          "for firewall group: %(fwg_id)s"),
-                      {'fwg_id': firewall_group['id']})
+            LOG.exception("FWaaS driver error in delete_firewall_group "
+                          "for firewall group: %s", firewall_group['id'])
             status = nl_constants.ERROR
 
         # Notify plugin of deletion or return firewall group's status to
@@ -523,11 +525,13 @@ class FWaaSL3AgentExtension(l3_extension.L3AgentExtension):
                 self.fwplugin_rpc.set_firewall_group_status(context,
                         firewall_group['id'], status)
         except Exception:
-            LOG.exception(_LE("FWaaS RPC failure in delete_firewall_group "
-                              "for firewall group: %(fwg_id)s"),
-                    {'fwg_id': firewall_group['id']})
+            LOG.exception("FWaaS RPC failure in delete_firewall_group "
+                          "for firewall group: %s", firewall_group['id'])
             self.services_sync_needed = True
 
+    def ha_state_change(self, context, data):
+        pass
+
 
 class L3WithFWaaS(FWaaSL3AgentExtension):
 
