@@ -94,7 +94,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             if router_info.snat_iptables_manager:
                 ipt_mgrs.append({'ipt': router_info.snat_iptables_manager,
                                  'if_prefix': SNAT_INT_DEV_PREFIX})
-        if router_info.dist_fip_count:
+        if router_info.rtr_fip_connect:
             # handle the fip case on n/w or compute node.
             ipt_mgrs.append({'ipt': router_info.iptables_manager,
                              'if_prefix': ROUTER_2_FIP_DEV_PREFIX})
@@ -363,9 +363,9 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
 
         # iptables adds '-m protocol' when any source
         # or destination port number is specified
-        if not((rule.get('source_port') is None)
-           and (rule.get('destination_port') is None)):
-                args += self._match_arg(rule.get('protocol'))
+        if (rule.get('source_port') is not None or
+            rule.get('destination_port') is not None):
+            args += self._match_arg(rule.get('protocol'))
 
         args += self._port_arg('sport',
                                rule.get('protocol'),
@@ -414,8 +414,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         return args
 
     def _port_arg(self, direction, protocol, port):
-        if (protocol not in ['udp', 'tcp']
-            or port is None):
+        if protocol not in ['udp', 'tcp'] or port is None:
             return []
 
         args = ['--%s' % direction, '%s' % port]
