@@ -98,7 +98,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             if ri.snat_iptables_manager:
                 ipt_mgrs.append({'ipt': ri.snat_iptables_manager,
                                  'if_prefix': SNAT_INT_DEV_PREFIX})
-        if ri.dist_fip_count:
+        if ri.rtr_fip_connect:
             # handle the fip case on n/w or compute node.
             ipt_mgrs.append({'ipt': ri.iptables_manager,
                              'if_prefix': ROUTER_2_FIP_DEV_PREFIX})
@@ -394,8 +394,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
 
         # iptables adds '-m protocol' when any source
         # or destination port number is specified
-        if not((rule.get('source_port') is None)
-           and (rule.get('destination_port') is None)):
+        if (rule.get('source_port') is not None or
+            rule.get('destination_port') is not None):
                 args += self._match_arg(rule.get('protocol'))
 
         args += self._port_arg('sport',
@@ -445,8 +445,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
         return args
 
     def _port_arg(self, direction, protocol, port):
-        if (protocol not in ['udp', 'tcp']
-            or port is None):
+        if protocol not in ['udp', 'tcp'] or port is None:
             return []
 
         args = ['--%s' % direction, '%s' % port]
