@@ -546,9 +546,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 dl_type=constants.ETHERTYPE_IPV6,
                 nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
                 icmp_type=icmp_type,
-                actions='resubmit(,%d)' % (
-                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
-            )
+                actions='normal')
 
     # NOTE(ivasilevskaya) That's a copy-paste from neutron ovsfw driver
     # which differs in constants (table numbers) and exception classes
@@ -584,8 +582,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
             table=fwaas_ovs_consts.FW_ACCEPT_OR_INGRESS_TABLE,
             priority=80,
             reg_port=ovs_port.ofport,
-            actions='resubmit(,%d)' % (
-                ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
+            actions='normal',
         )
 
     # NOTE(ivasilevskaya) That's a copy-paste from neutron ovsfw driver
@@ -624,8 +621,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 dl_src=mac_addr,
                 dl_type=constants.ETHERTYPE_ARP,
                 arp_spa=ip_addr,
-                actions='resubmit(,%d)' % (
-                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
+                actions='normal'
             )
             self._add_flow(
                 table=fwaas_ovs_consts.FW_BASE_EGRESS_TABLE,
@@ -748,8 +744,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 table=fwaas_ovs_consts.FW_ACCEPT_OR_INGRESS_TABLE,
                 priority=80,
                 reg_port=port.ofport,
-                actions='resubmit(,%d)' % (
-                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
+                actions='normal'
             )
 
     # NOTE(ivasilevskaya) That's a copy-paste from neutron ovsfw driver
@@ -782,8 +777,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 ct_mark=fwaas_ovs_consts.CT_MARK_NORMAL,
                 reg_port=port.ofport,
                 ct_zone=port.vlan_tag,
-                actions='resubmit(,%d)' % (
-                    ovs_consts.ACCEPTED_EGRESS_TRAFFIC_TABLE)
+                actions='normal'
             )
         self._add_flow(
             table=fwaas_ovs_consts.FW_RULES_EGRESS_TABLE,
@@ -817,9 +811,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 dl_type=constants.ETHERTYPE_IPV6,
                 nw_proto=lib_const.PROTO_NUM_IPV6_ICMP,
                 icmp_type=icmp_type,
-                actions='output:{:d},resubmit(,{:d})'.format(
-                    port.ofport,
-                    ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE),
+                actions='output:{:d}'.format(port.ofport)
             )
 
     # NOTE(ivasilevskaya) That's a copy-paste from neutron ovsfw driver
@@ -831,9 +823,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
             priority=100,
             dl_type=constants.ETHERTYPE_ARP,
             reg_port=port.ofport,
-            actions='output:{:d},resubmit(,{:d})'.format(
-                port.ofport,
-                ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE),
+            actions='output:{:d}'.format(port.ofport)
         )
         self._initialize_ingress_ipv6_icmp(port)
 
@@ -849,9 +839,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 nw_proto=lib_const.PROTO_NUM_UDP,
                 tp_src=src_port,
                 tp_dst=dst_port,
-                actions='output:{:d},resubmit(,{:d})'.format(
-                    port.ofport,
-                    ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE),
+                actions='output:{:d}'.format(port.ofport)
             )
 
         # Track untracked
@@ -904,9 +892,7 @@ class OVSFirewallDriver(driver_base.FirewallL2DriverBase):
                 ct_state=state,
                 ct_mark=fwaas_ovs_consts.CT_MARK_NORMAL,
                 ct_zone=port.vlan_tag,
-                actions='output:{:d},resubmit(,{:d})'.format(
-                    port.ofport,
-                    ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE)
+                actions='output:{:d}'.format(port.ofport)
             )
         self._add_flow(
             table=fwaas_ovs_consts.FW_RULES_INGRESS_TABLE,
