@@ -87,9 +87,7 @@ def populate_flow_common(direction, flow_template, port):
     """Initialize common flow fields."""
     if direction == n_consts.INGRESS_DIRECTION:
         flow_template['table'] = fwaas_ovs_consts.FW_RULES_INGRESS_TABLE
-        flow_template['actions'] = "output:{:d},resubmit(,{:d})".format(
-            port.ofport,
-            ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE)
+        flow_template['actions'] = "output:{:d}".format(port.ofport)
     elif direction == n_consts.EGRESS_DIRECTION:
         flow_template['table'] = fwaas_ovs_consts.FW_RULES_EGRESS_TABLE
         # Traffic can be both ingress and egress, check that no ingress rules
@@ -189,8 +187,11 @@ def create_accept_flows(flow, sg_enabled=False):
         resubmit_to_sg(flow)
     elif flow['table'] == fwaas_ovs_consts.FW_RULES_INGRESS_TABLE:
         flow['actions'] = (
-            'ct(commit,zone=NXM_NX_REG{:d}[0..15]),{:s}'.format(
-                fwaas_ovs_consts.REG_NET, flow['actions']))
+            'ct(commit,zone=NXM_NX_REG{:d}[0..15]),{:s},'
+            'resubmit(,{:d})'.format(
+                fwaas_ovs_consts.REG_NET, flow['actions'],
+                ovs_consts.ACCEPTED_INGRESS_TRAFFIC_TABLE)
+        )
     result.append(flow)
     return result
 
