@@ -66,6 +66,14 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
                                                  protocol="tcp")
         self.fw_policy = self.create_firewall_policy()
 
+    def _try_delete_router(self, router):
+        # delete router, if it exists
+        try:
+            self.delete_router(router)
+        # if router is not found, this means it was deleted in the test
+        except lib_exc.NotFound:
+            pass
+
     def _try_delete_policy(self, policy_id):
         # delete policy, if it exists
         try:
@@ -106,8 +114,24 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
                  (fw_id, target_states))
             raise lib_exc.TimeoutException(m)
 
+    def _wait_until_deleted(self, fw_id):
+        def _wait():
+            try:
+                firewall = self.firewalls_client.show_firewall(fw_id)
+            except lib_exc.NotFound:
+                return True
+
+            fw_status = firewall['firewall']['status']
+            if fw_status == 'ERROR':
+                raise lib_exc.DeleteErrorException(resource_id=fw_id)
+
+        if not test_utils.call_until_true(_wait, CONF.network.build_timeout,
+                                          CONF.network.build_interval):
+            m = ("Timed out waiting for firewall %s deleted" % fw_id)
+            raise lib_exc.TimeoutException(m)
+
     @decorators.idempotent_id('1b84cf01-9c09-4ce7-bc72-b15e39076468')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_firewall_rules(self):
         # List firewall rules
         fw_rules = self.firewall_rules_client.list_firewall_rules()
@@ -126,7 +150,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
                         m['enabled']) for m in fw_rules])
 
     @decorators.idempotent_id('563564f7-7077-4f5e-8cdc-51f37ae5a2b9')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_update_delete_firewall_rule(self):
         # Create firewall rule
         body = self.firewall_rules_client.create_firewall_rule(
@@ -134,6 +158,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
             action="allow",
             protocol="tcp")
         fw_rule_id = body['firewall_rule']['id']
+        self.addCleanup(self._try_delete_rule, fw_rule_id)
 
         # Update firewall rule
         body = self.firewall_rules_client.update_firewall_rule(fw_rule_id,
@@ -148,7 +173,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
                          [m['id'] for m in fw_rules['firewall_rules']])
 
     @decorators.idempotent_id('3ff8c08e-26ff-4034-ae48-810ed213a998')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_show_firewall_rule(self):
         # show a created firewall rule
         fw_rule = self.firewall_rules_client.show_firewall_rule(
@@ -157,7 +182,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
             self.assertEqual(self.fw_rule[key], value)
 
     @decorators.idempotent_id('1086dd93-a4c0-4bbb-a1bd-6d4bc62c199f')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_list_firewall_policies(self):
         fw_policies = self.firewall_policies_client.list_firewall_policies()
         fw_policies = fw_policies['firewall_policies']
@@ -169,7 +194,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
                         m['firewall_rules']) for m in fw_policies])
 
     @decorators.idempotent_id('bbf37b6c-498c-421e-9c95-45897d3ed775')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_update_delete_firewall_policy(self):
         # Create firewall policy
         body = self.firewall_policies_client.create_firewall_policy(
@@ -192,7 +217,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
         self.assertNotIn(fw_policy_id, [m['id'] for m in fw_policies])
 
     @decorators.idempotent_id('1df59b3a-517e-41d4-96f6-fc31cf4ecff2')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_show_firewall_policy(self):
         # show a created firewall policy
         fw_policy = self.firewall_policies_client.show_firewall_policy(
@@ -202,7 +227,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
             self.assertEqual(self.fw_policy[key], value)
 
     @decorators.idempotent_id('02082a03-3cdd-4789-986a-1327dd80bfb7')
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     def test_create_show_delete_firewall(self):
         # Create tenant network resources required for an ACTIVE firewall
         network = self.create_network()
@@ -210,6 +235,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
         router = self.create_router(
             data_utils.rand_name('router-'),
             admin_state_up=True)
+        self.addCleanup(self._try_delete_router, router)
         self.routers_client.add_router_interface(router['id'],
                                                  subnet_id=subnet['id'])
 
@@ -246,15 +272,25 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
         # Delete firewall
         self.firewalls_client.delete_firewall(firewall_id)
 
+        # Wait for the firewall resource to be deleted
+        self._wait_until_deleted(firewall_id)
+
+        # Confirm deletion
+        firewalls = self.firewalls_client.list_firewalls()['firewalls']
+        self.assertNotIn(firewall_id, [m['id'] for m in firewalls])
+
     @decorators.idempotent_id('1355cf5c-77d4-4bb9-87d7-e50c194d08b5')
     def test_firewall_insertion_mode_add_remove_router(self):
         # Create legacy routers
         router1 = self.create_router(
             data_utils.rand_name('router-'),
             admin_state_up=True)
+        self.addCleanup(self._try_delete_router, router1)
         router2 = self.create_router(
             data_utils.rand_name('router-'),
             admin_state_up=True)
+        self.addCleanup(self._try_delete_router, router2)
+
         # Create firewall on a router1
         body = self.firewalls_client.create_firewall(
             name=data_utils.rand_name("firewall"),
@@ -298,6 +334,7 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
         router = self.create_router(
             data_utils.rand_name('router1-'),
             admin_state_up=True)
+        self.addCleanup(self._try_delete_router, router)
 
         # Create firewall
         body = self.firewalls_client.create_firewall(
@@ -308,15 +345,15 @@ class FWaaSExtensionTestJSON(base.BaseFWaaSTest):
         self.addCleanup(self._try_delete_firewall, created_firewall['id'])
 
         # Try to create firewall with the same router
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
             lib_exc.Conflict,
-            "already associated with other Firewall",
+            "already associated with other firewall",
             self.firewalls_client.create_firewall,
             name=data_utils.rand_name("firewall"),
             firewall_policy_id=self.fw_policy['id'],
             router_ids=[router['id']])
 
-    @test.attr(type='smoke')
+    @decorators.attr(type='smoke')
     @decorators.idempotent_id('53305b4b-9897-4e01-87c0-2ae386083180')
     def test_firewall_rule_insertion_position_removal_rule_from_policy(self):
         # Create firewall rule
