@@ -18,8 +18,12 @@ import contextlib
 import mock
 from neutron.api import extensions as api_ext
 from neutron.common import config
-from neutron import context
-from neutron.extensions import l3
+from neutron_lib.api.definitions import firewall
+from neutron_lib import constants as nl_constants
+from neutron_lib import context
+from neutron_lib.exceptions import firewall_v1 as f_exc
+from neutron_lib.exceptions import l3
+from neutron_lib.plugins import directory
 from oslo_config import cfg
 from oslo_utils import importutils
 from oslo_utils import uuidutils
@@ -28,11 +32,9 @@ import webob.exc
 
 from neutron_fwaas.db.firewall import firewall_db as fdb
 from neutron_fwaas import extensions
-from neutron_fwaas.extensions import firewall
 from neutron_fwaas.services.firewall import fwaas_plugin
 from neutron_fwaas.tests import base
-from neutron_lib import constants as nl_constants
-from neutron_lib.plugins import directory
+
 
 DB_FW_PLUGIN_KLASS = (
     "neutron_fwaas.db.firewall.firewall_db.Firewall_db_mixin"
@@ -73,7 +75,7 @@ class FakeAgentApi(fwaas_plugin.FirewallCallbacks):
 
 class FirewallPluginDbTestCase(base.NeutronDbPluginV2TestCase):
     resource_prefix_map = dict(
-        (k, firewall.FIREWALL_PREFIX)
+        (k, firewall.API_PREFIX)
         for k in firewall.RESOURCE_ATTRIBUTE_MAP.keys()
     )
 
@@ -86,7 +88,7 @@ class FirewallPluginDbTestCase(base.NeutronDbPluginV2TestCase):
         service_plugins = {'fw_plugin_name': fw_plugin}
 
         fdb.Firewall_db_mixin.supported_extension_aliases = ["fwaas"]
-        fdb.Firewall_db_mixin.path_prefix = firewall.FIREWALL_PREFIX
+        fdb.Firewall_db_mixin.path_prefix = firewall.API_PREFIX
         super(FirewallPluginDbTestCase, self).setUp(
             ext_mgr=ext_mgr,
             service_plugins=service_plugins
@@ -627,7 +629,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
             req = self.new_delete_request('firewall_policies', fwp_id)
             res = req.get_response(self.ext_api)
             self.assertEqual(204, res.status_int)
-            self.assertRaises(firewall.FirewallPolicyNotFound,
+            self.assertRaises(f_exc.FirewallPolicyNotFound,
                               self.plugin.get_firewall_policy,
                               ctx, fwp_id)
 
@@ -650,7 +652,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                 req = self.new_delete_request('firewall_policies', fwp_id)
                 res = req.get_response(self.ext_api)
                 self.assertEqual(204, res.status_int)
-                self.assertRaises(firewall.FirewallPolicyNotFound,
+                self.assertRaises(f_exc.FirewallPolicyNotFound,
                                   self.plugin.get_firewall_policy,
                                   ctx, fwp_id)
                 fw_rule = self.plugin.get_firewall_rule(ctx, fr_id)
@@ -980,7 +982,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
             req = self.new_delete_request('firewall_rules', fwr_id)
             res = req.get_response(self.ext_api)
             self.assertEqual(204, res.status_int)
-            self.assertRaises(firewall.FirewallRuleNotFound,
+            self.assertRaises(f_exc.FirewallRuleNotFound,
                               self.plugin.get_firewall_rule,
                               ctx, fwr_id)
 
@@ -1196,7 +1198,7 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                 req = self.new_delete_request('firewalls', fw_id)
                 res = req.get_response(self.ext_api)
                 self.assertEqual(204, res.status_int)
-                self.assertRaises(firewall.FirewallNotFound,
+                self.assertRaises(f_exc.FirewallNotFound,
                                   self.plugin.get_firewall,
                                   ctx, fw_id)
 
@@ -1406,8 +1408,8 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
                 name='firewall_policy2', firewall_rules=[associated]) as fwp:
                 fwp_id = fwp['firewall_policy']['id']
                 not_associated = fwr2['firewall_rule']['id']
-                msg = "Firewall Rule {0} is not associated with " \
-                      "Firewall Policy {1}.".format(not_associated, fwp_id)
+                msg = "Firewall rule {0} is not associated with " \
+                      "firewall policy {1}.".format(not_associated, fwp_id)
                 result = self._rule_action(
                     'remove', fwp_id, not_associated,
                     insert_before=None,
@@ -1517,3 +1519,9 @@ class TestFirewallDBPlugin(FirewallPluginDbTestCase):
         with self.firewall(name='fireWall1') as fw:
             res = self._show('firewalls', fw['firewall']['id'])
             self.assertEqual('fireWall1', res['firewall']['name'])
+
+    def test_create_firewall_rule_with_invalid_action_type(self):
+        attrs = self._get_test_firewall_rule_attrs()
+        attrs['action'] = 123
+        res = self._create_firewall_rule(self.fmt, **attrs)
+        self.assertEqual(400, res.status_int)
